;             2000 Access I/O Processor Load and Dump Command File.
;
;
; This command file is invoked to run as a separate simulator instance to load
; a new 2000 Access I/O Processor operating system into the IOP via a cross-load
; from the System Processor and then to dump it back to the SP to be stored to
; magnetic tape.  It is invoked by the "tsb-iopgen.sim" command file, as
; follows:
;
;    hp2100 iopdump.inc <cpu-model> <cpu-series>
;
; ...where <cpu-model> is the specific CPU model to simulate (2100, 21MX, or
; 21MX-E), and <cpu-series> is the general CPU type (2100 or 21MX).  It is never
; invoked directly by the user.
;
;
; NOTES:
;
;  - This command file uses the IPL interlock capability introduced in Release
;    30 of the HP2100 simulator; consequently, it will not run with earlier
;    releases.  Interlocking is required during the transfer of the IOP program
;    from the SP and again during transfer of the device table from the IOP to
;    the SP during system startup.  If the simulator's host platform does not
;    support semaphores, the "alt-*.sim" command files should be used, as these
;    employ a fallback mechanism that should allow reasonably reliable
;    operation.


; Capture the I/O Processor load and dump log.

set -N console log=iopdump-%2.log


; Configure the I/O Processor.

do iopconfig.inc %1


; Connect the Processor Interconnect cables to the IOP.
;
; The code number here ("4020") is significant only in that the SP instance
; must use the same code number to ensure interprocessor communication.

attach -I IPL 4020


; Load and start the binary loader, which is used as a cross-loader via the IPL
; device.  The loader starts with a CLC 0,C that must execute before the SP
; begins to send data.  With interlocking enabled, this is guaranteed, because
; the teletype response of a line feed by the SP will take a minimum of 200
; machine instructions, while the CLC 0,C is executed by the IOP in a maximum
; of 10 instructions.  As long as interlocking level is set for 200 or fewer
; instructions, the requirement will be satisfied.
;
; In hardware, an HP 21xx IOP uses the BBL (Basic Binary Loader), whereas an HP
; 1000 IOP uses the paper tape loader ROM (12992K).  In either case, the loader
; is configured to read from the lower processor interconnect select code.
;
; In simulation, we use the IPL device loader, which automatically uses the
; correct loader for the current CPU model (actually, it uses the 2000 Access
; modified BBL for 21xx CPUs).  This permits changing the CPU model above
; without having to redo this section.

echo Receiving the IOP program.

deposit S 000000
boot IPL


; Verify that we had a successful cross-load.

assert T=102077

echo
echo IOP cross-loading completed successfully.


; The IOP is now loaded.  It must now be unloaded back to the SP.  This cannot
; be done until the SP prints "START IOP AT LOCATION 2002" to indicate that it
; is ready.  If we don't wait, then the IOP-to-SP transfer will not occur
; because the SP issues a CLF to initialize the IPL just before printing the
; message.  If the IOP sets the flag before the SP clears it, the SP and IOP
; both hang while waiting for the IPL transfer.
;
; The SP recognizes that the IOP loader has stopped when a transfer is not
; acknowledged by the IOP.  The resulting timeout won't occur unless we release
; the interlock while the IOP is halted, so the SP can execute freely.

set IPL INTERLOCK=0


; Wait for the SP to time out after cross-loading completes.

echo
echo Waiting for the SP to time out after completing the cross-load...

set IPL WAIT


; The SP is now ready for us.  Start the IOP to dump memory across the IPL.
;
; Expect halt 102077 when the transfer is complete.

echo
echo Dumping the IOP program back to the SP.

deposit P 002002
go


; The IOP tape is now written.  Then the memory map is printed, and finally the
; CPU halts with 77B.
;
; The IOP also halts after the tape is written.  However, the IOP must not exit
; at that point, or the SP's IPL poll will fail.

assert T=102077

echo
echo IOP dumping completed successfully.


; We have to wait for the SP to halt before we can exit.  However, we must
; release the interlock while the IOP is halted, so the SP can execute freely
; to complete its execution.

set IPL INTERLOCK=0


; Wait until the SP is done before exiting.

echo
echo Waiting for the SP to complete and halt.

set IPL WAIT


; We're done.

exit
